@@ -1,4 +1,4 @@
-/* @(#)pd/sdbm/tune.h	1.1 5/12/92 11:03:55 */
+/* @(#) $Id: tune.h,v 1.1 1992/05/12 11:03:55 tron Exp $ */
 /*
  * sdbm - ndbm work-alike hashed database library
  * tuning and portability constructs [not nearly enough]
@@ -34,7 +34,7 @@ extern void free();
 #ifdef POSIX_OS
 #include <unistd.h>
 #else
-extern long lseek();
+/* extern long lseek(); */
 #endif
 
 /*
