@@ -4,5 +4,5 @@
  * expand.c to define the values for $compile_num and $compile_date.
  * To reset the compilation count, simply remove ldinfo.c.
  */
-int compile_num = 4;
-char *compile_date = "30-may-94";
+int compile_num = 7;
+char *compile_date = "28-mar-95";
