@@ -1,4 +1,4 @@
-/* @(#)src/lookup.c	1.14 9/20/92 18:45:55 */
+/* @(#) $Id: lookup.c,v 1.15 1994/10/26 14:14:22 nm4 Exp $ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -39,6 +39,9 @@
 #ifdef HAVE_NIALIAS
 # include <aliasdb.h>
 #endif
+#ifdef  HAVE_NISPLUS
+# include <rpcsvc/nis.h>
+#endif
 #ifndef DEPEND
 # include "extern.h"
 # include "debug.h"
@@ -87,6 +90,12 @@ static void aliasni_close();
 static int aliasni_lookup();
 #endif
 
+#ifdef  HAVE_NISPLUS
+static int nisplus_open();
+static void nisplus_close();
+static int nisplus_lookup();
+#endif
+
 
 /*
  * the following structure defines the available access methods.
@@ -114,6 +123,10 @@ static struct proto {
     { "nialias",			/* NetInfo remote alias search */
       aliasni_open, aliasni_close, aliasni_lookup },
 #endif /* HAVE_NIALIAS */
+#ifdef  HAVE_NISPLUS
+    { "nisplus",			/* NIS+ remote database search */
+      nisplus_open, nisplus_close, nisplus_lookup },
+#endif /* HAVE_NISPLUS */
 };
 
 /* point to end of protos table */
@@ -1212,6 +1225,355 @@ aliasni_lookup(db, key, value, error)
 
 #endif /* HAVE_NIALIAS */
 
+#ifdef  HAVE_NISPLUS
+/*
+ * NIS+ remote access method
+ * (This is NIS version 3
+ * for NIS version 2, it's called Yellow Pages so use HAVE_YP)
+ *
+ * NIS+ tables can be partially or fully qualified.  For example:
+ *   mail_aliases.org_dir
+ *   mail_aliases.org_dir.oes
+ *   mail_aliases.org_dir.oes.amdahl.com.
+ * The trailing dot "." indicates a fully-qualified NIS+ name
+ *
+ * If the system is properly configured, the server, domain, and search path
+ * should already be established external to smail.  There may even be
+ * mirrors and caches, all of which are transparent unless specifically
+ * disabled.  (We won't disable them.)
+ */
+
+/* private data structure */
+struct nisplus_db {
+    struct proto	*proto;
+    char		*table;
+    int			fieldnum;
+    int			retries,	/* saved retry/interval for lookup */
+			interval;
+};
+
+/* open a NIS+ database */
+static int
+nisplus_open(name, proto, retries, interval, statp, db, error)
+    char *name;				/* name of file */
+    struct proto *proto;		/* access method */
+    int retries;			/* retry count */
+    int interval;			/* retry interval */
+    struct stat *statp;			/* save stat results here */
+    char **db;				/* store open database info here */
+    char **error;			/* store error message here */
+{
+    register struct nisplus_db	*priv;
+    nis_result			*result;
+    int				flags;
+    char			*domain;
+
+    priv = (struct nisplus_db *)xmalloc(sizeof(*priv));
+    priv->proto = proto;
+    priv->fieldnum = 1;
+    priv->retries = retries;
+    priv->interval = interval;
+
+    /* check for directive to change field number */
+    /* we use @ in character 0 because NIS+ strings cannot start with @ */
+    if ( name[0] == '@' ) {
+	char	*offset;
+
+        priv->fieldnum = atoi(name+1);
+	offset = index(name,',');
+	if ( offset ) {
+            priv->table = strdup(offset+1);
+        } else {
+	    /* Yech!  No commas!  This is a failure condition */
+	    *error = "syntax error in name given to nisplus proto";
+	    xfree(priv);
+	    return FILE_FAIL;
+	}
+    } else {
+        priv->fieldnum = 1;
+        priv->table = name;
+    }
+
+    /* if stat required, just zero it out, there is nothing to put there */
+    if (statp) {
+	(void) bzero((char *)statp, sizeof(*statp));
+    }
+
+    /* find offset to domain name, advance past the last comma */
+    if ( !(domain = rindex(name,','))) {
+	domain = name;
+    } else {
+	domain++;
+    }
+
+    DEBUG1(DBG_DRIVER_HI, "nisplus_open: verifying <%s>\n", domain);
+
+    /* we'll just check if the table exists and then proceed */
+    flags = FOLLOW_LINKS | EXPAND_NAME;
+    do {
+	result = nis_lookup ( domain, flags );
+	switch ( result->status ) {
+
+	    /* success - FILE_SUCCEED */
+	    case NIS_SUCCESS:
+	    case NIS_S_SUCCESS:
+		/* we found it, but is it a table? */
+		if ( result->objects.objects_len > 1 ) {
+		    *error = "multiple results - NIS+ table name is ambiguous";
+		    nis_freeresult(result);
+		    DEBUG(DBG_DRIVER_HI, "nisplus_open: more than 1 value\n");
+	            xfree(priv);
+		    return FILE_FAIL;
+		}
+		if ( result->objects.objects_val[0].zo_data.zo_type != TABLE_OBJ ) {
+		    *error = "table open attempted on non-table NIS+ object";
+		    nis_freeresult(result);
+		    DEBUG(DBG_DRIVER_HI, "nisplus_open: not a table\n");
+	            xfree(priv);
+		    return FILE_FAIL;
+		}
+
+		nis_freeresult(result);
+		*db = (char*) priv;
+	        DEBUG1(DBG_DRIVER_HI, "nisplus_open: found <%s>\n", domain);
+		return FILE_SUCCEED;
+
+	    /* immediate retry on cache expiration */
+	    case NIS_CACHEEXPIRED:
+		flags |= NO_CACHE;
+		break;
+
+	    /* immediate retry on server resource exhaustion */
+	    case NIS_TRYAGAIN:
+	    case NIS_NOMEMORY:
+		break;
+
+	    /* failure - not found - FILE_NOMATCH */
+	    case NIS_NOTFOUND:
+	    case NIS_NOSUCHNAME:
+	    case NIS_NOSUCHTABLE:
+	    case NIS_BADNAME:
+	    case NIS_LINKNAMEERROR:
+	    case NIS_FOREIGNNS:
+		*error = nis_sperror(result->status, "nisplus_open");
+		nis_freeresult(result);
+	        DEBUG1(DBG_DRIVER_HI, "nisplus_open: <%s> not found\n", domain);
+	        xfree(priv);
+		return FILE_NOMATCH;
+
+	    /* failure - try again later - FILE_AGAIN */
+	    case NIS_NAMEUNREACHABLE:
+	    case NIS_SYSTEMERROR:
+	    case NIS_NOT_ME:
+	    case NIS_RPCERROR:
+		*error = nis_sperror(result->status, "nisplus_open");
+		nis_freeresult(result);
+	        DEBUG1(DBG_DRIVER_HI, "nisplus_open: try again <%s>\n", domain);
+	        xfree(priv);
+		return FILE_AGAIN;
+
+	    /* failure - permanent failure on this object - FILE_FAIL */
+	    case NIS_UNKNOWNOBJ:
+	    case NIS_INVALIDOBJ:
+	    default:
+		*error = nis_sperror(result->status, "nisplus_open");
+		nis_freeresult(result);
+	        DEBUG1(DBG_DRIVER_HI, "nisplus_open: <%s> not found\n", domain);
+	        xfree(priv);
+		return FILE_FAIL;
+	}
+
+	/* sleep interval for retry */
+	if ( retries > 0 ) {
+	    sleep ( interval );
+	}
+    } while ( retries-- > 0 );
+
+    /* if we get here, we ran out of immediate retries */
+    /* the director should be told to try again later */
+    *error = nis_sperror(result->status, "nisplus_open");
+    nis_freeresult(result);
+    DEBUG1(DBG_DRIVER_HI, "nisplus_open: try again <%s>\n", domain);
+    xfree(priv);
+    return FILE_AGAIN;
+}
+
+/* close a NIS+ database */
+static void
+nisplus_close(db)
+    struct nisplus_db *db;
+{
+    xfree((char *)db);
+    return;
+}
+
+/* look up an element of a NIS+ database */
+/*ARGSUSED*/
+static int
+nisplus_lookup(db, key, value, error)
+    register struct nisplus_db *db;
+    char *key;
+    char **value;
+    char **error;
+{
+    nis_result			*result;
+    struct nis_object		*objbuf;
+    int				flags,
+				col,
+				retries,
+				valsize;
+    char			*name,
+				*valbuf;
+
+    valsize=0;
+    retries = db->retries;
+
+    /* make the name to look up */
+    /* prepend the search string as a "subdomain" of the table */
+    name = (char*) xmalloc(strlen(key)+strlen(db->table)+2);
+    sprintf(name,db->table,key);
+
+    DEBUG1(DBG_DRIVER_HI, "nisplus_lookup: looking for <%s>\n", key);
+    DEBUG1(DBG_DRIVER_HI, "nisplus_lookup: asking NIS+ for <%s>\n", name);
+
+    /* look up the object */
+    flags = FOLLOW_LINKS | EXPAND_NAME | RETURN_RESULT;
+    do {
+	result = nis_list ( name, flags, NULL, NULL );
+	switch ( result->status ) {
+
+	    /* success - DB_SUCCEED */
+	    /* with two exceptions... */
+	    case NIS_SUCCESS:
+	    case NIS_S_SUCCESS:
+		/* OK, we found it, but did we find too many? */
+		if ( result->objects.objects_len > 1 ) {
+		    *error = "multiple results - NIS+ entry name is ambiguous";
+		    nis_freeresult(result);
+		    DEBUG(DBG_DRIVER_HI, "nisplus_lookup: more than 1 value\n");
+		    xfree(name);
+		    return DB_FAIL;
+		}
+
+		/* ...but did we really find anything? */
+		if ( result->objects.objects_len < 1 ) {
+		    *error = "empty results - NIS+ search returned nothing";
+		    nis_freeresult(result);
+		    DEBUG(DBG_DRIVER_HI, "nisplus_lookup: returned nothing\n");
+		    xfree(name);
+		    return DB_FAIL;
+		}
+
+		/* ...but is it a table entry? */
+		if ( result->objects.objects_val[0].zo_data.zo_type != ENTRY_OBJ ) {
+		    *error = "open attempted on non-entry NIS+ object";
+		    nis_freeresult(result);
+		    DEBUG(DBG_DRIVER_HI, "nisplus_lookup: not a table entry\n");
+		    xfree(name);
+		    return DB_FAIL;
+		}
+
+		/* ...but are there enough fields? */
+		if ( result->objects.objects_val[0].zo_data.objdata_u.en_data.en_cols.en_cols_len <= db->fieldnum ) {
+		    *error = "NIS+ result had too few fields for the request";
+		    nis_freeresult(result);
+		    DEBUG(DBG_DRIVER_HI, "nisplus_lookup: not enough fields\n");
+		    xfree(name);
+		    return DB_FAIL;
+		}
+
+		/* assemble the return value from the columns */
+		objbuf = &(result->objects.objects_val[0]);
+		valbuf = xmalloc(objbuf->zo_data.objdata_u.en_data.en_cols.en_cols_val[db->fieldnum].ec_value.ec_value_len+1);
+		strncpy(valbuf, objbuf->zo_data.objdata_u.en_data.en_cols.en_cols_val[db->fieldnum].ec_value.ec_value_val,
+				objbuf->zo_data.objdata_u.en_data.en_cols.en_cols_val[db->fieldnum].ec_value.ec_value_len);
+		valsize = objbuf->zo_data.objdata_u.en_data.en_cols.en_cols_val[db->fieldnum].ec_value.ec_value_len+1;
+
+		/* back up over any trailing whitespace */
+		for (valsize--; valsize > 0 && isspace(valbuf[valsize]); valsize--);
+		    valsize++;
+		    if (isspace(valbuf[valsize])) {
+			valbuf[valsize] = '\0';
+		}
+
+		DEBUG1(DBG_DRIVER_HI, "nisplus_lookup: got <%s>\n", valbuf);
+		*value=valbuf;
+		nis_freeresult(result);
+		xfree(name);
+		return DB_SUCCEED;
+
+	    /* immediate retry on cache expiration */
+	    case NIS_CACHEEXPIRED:
+		flags |= NO_CACHE;
+		break;
+
+	    /* immediate retry on server resource exhaustion */
+	    case NIS_TRYAGAIN:
+	    case NIS_NOMEMORY:
+		break;
+
+	    /* failure - not found - DB_NOMATCH */
+	    case NIS_NOTFOUND:
+	    case NIS_S_NOTFOUND:
+	    case NIS_NOSUCHNAME:
+	    case NIS_NOSUCHTABLE:
+	    case NIS_BADNAME:
+	    case NIS_LINKNAMEERROR:
+	    case NIS_FOREIGNNS:
+	    case NIS_PARTIAL:
+		*error = nis_sperror(result->status, "nisplus_lookup");
+		nis_freeresult(result);
+		DEBUG2(DBG_DRIVER_HI, "nisplus_lookup: <%s> not found: %s\n",
+		    key, *error);
+		xfree(name);
+		return DB_NOMATCH;
+
+	    /* failure - try again later - FILE_AGAIN */
+	    case NIS_NAMEUNREACHABLE:
+	    case NIS_SYSTEMERROR:
+	    case NIS_NOT_ME:
+	    case NIS_RPCERROR:
+		*error = nis_sperror(result->status, "nisplus_lookup");
+		nis_freeresult(result);
+		DEBUG2(DBG_DRIVER_HI, "nisplus_lookup: try again <%s>: %s\n",
+		    key, *error);
+		xfree(name);
+		return FILE_AGAIN;
+
+	    /* failure - permanent failure on this object - DB_FAIL */
+	    case NIS_UNKNOWNOBJ:
+	    case NIS_INVALIDOBJ:
+	    case NIS_BADATTRIBUTE:
+	    case NIS_NOTSEARCHABLE:
+	    case NIS_TOOMANYATTRS:
+	    default:
+		*error = nis_sperror(result->status, "nisplus_lookup");
+		nis_freeresult(result);
+		DEBUG2(DBG_DRIVER_HI, "nisplus_lookup: <%s> not found: %s\n",
+		    key, *error);
+		xfree(name);
+		return DB_FAIL;
+	}
+
+	/* sleep interval for retry */
+	if ( retries > 0 ) {
+	    sleep ( db->interval );
+	}
+    } while ( retries-- > 0 );
+
+    /* if we get here, we ran out of immediate retries */
+    /* the director should be told to try again later */
+    *error = nis_sperror(result->status, "nisplus_lookup");
+    nis_freeresult(result);
+    DEBUG2(DBG_DRIVER_HI, "nisplus_lookup: try again <%s>: %s\n",
+	key, *error);
+    xfree(name);
+    return FILE_AGAIN;
+    
+}
+
+#endif /* HAVE_NISPLUS */
+
 #ifdef STANDALONE
 
 int debug = 0;
