@@ -1,4 +1,4 @@
-/* @(#)src/queue.c	1.22 9/20/92 12:55:03 */
+/* @(#) $Id: queue.c,v 1.26 1994/11/10 17:41:22 nm4 Exp $ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -282,6 +282,28 @@ queue_envelope(list)
 	}
     }
 
+    /*
+     * write the ident of the sending person if known
+     */
+    if (ident_sender) {
+	if (put_line("-oMu") == FAIL ||
+	    put_line(ident_sender) == FAIL)
+	{
+	    return FAIL;
+	}
+    }
+
+    /*
+     * write the method used to determine the ident of sender if known
+     */
+    if (ident_method) {
+	if (put_line("-oMv") == FAIL ||
+	    put_line(ident_method) == FAIL)
+	{
+	    return FAIL;
+	}
+    }
+
     /*
      * write out the invoked program, too
      */
@@ -454,7 +476,7 @@ static int
 fixup_login_user()
 {
     register int i;
-    extern long lseek();
+/*  extern long lseek();  */
     struct passwd *pw;
 
     if (local_sender)
@@ -1157,7 +1179,24 @@ log_incoming()
 	*p = '\0';
     }
 
-    write_log(LOG_SYS, "received%s%s%s%s%s%s%s%s%s%s%s%ld%s",
+
+#if (SMAIL_LOG_STYLE == 2)
+    /* Default new log style (as of 3.1.29) */
+    /* This is SMAIL_LOG_STYLE=2 */
+    write_log(LOG_SYS, "Received%s%s%s%s%s%s%s%s%s%s%s%ld%s%s%s%s",
+	      " FROM:", sender,
+	      host_string? " HOST:": "", host_string? host_string: "",
+	      sender_proto? " PROTOCOL:": "", sender_proto? sender_proto: "",
+	      " PROGRAM:", program,
+	      trim_old_id? " ORIG-ID:": "", trim_old_id? trim_old_id: "",
+	      " SIZE:", msg_size,
+	      ident_sender? " IDENT:": "", ident_sender? ident_sender: "",
+	      ident_method? " ID-METHOD:": "", ident_method? ident_method: "");
+#else
+#if (!defined(SMAIL_LOG_STYLE)) || (SMAIL_LOG_STYLE == 1)
+    /* Old log style (as used by 3.1.28) */
+    /* This is SMAIL_LOG_STYLE=1 */
+    write_log(LOG_SYS, "received%s%s%s%s%s%s%s%s%s%s%s%ld%s%s%s%s%s",
 	      "\n|\t     from: ", sender,
 		host_string?
 	      "\n|\t     host: ": "", host_string? host_string: "",
@@ -1166,8 +1205,15 @@ log_incoming()
 	      "\n|\t  program: ", program,
 		trim_old_id?
 	      "\n|\t  orig-id: ": "", trim_old_id? trim_old_id: "",
-	      "\n|\t     size: ", msg_size, " bytes");
-
+	      "\n|\t     size: ", msg_size, " bytes",
+	      ident_sender? 
+	      "\n|\t    ident: ": "", ident_sender? ident_sender: "",
+	      ident_method? 
+	      "\n|\t ident-by: ": "", ident_method? ident_method: "");
+#else
+	ERROR -  SMAIL_LOG_STYLE set to invalid value.
+#endif
+#endif
     if (host_string) {
 	xfree(host_string);
     }
@@ -1209,11 +1255,22 @@ scan_spool_dirs()
 	char *in_dn = xprintf("%s/input", dn);
 
 	for (fn = scan_dir(in_dn); fn; fn = scan_dir((char *)NULL)) {
+	    int entry_grade;
+
 	    /* see if the file matches the form of a spool file */
 	    if (! isdigit(fn[0]) || strlen(fn) != SPOOL_FN_LEN) {
 		continue;		/* nope */
 	    }
 
+	    /* See if the grade is within the range being processed */
+	    /* But do not do this if printing the queue */
+	    if (operation_mode != PRINT_QUEUE) {
+		entry_grade =  fn[strlen(fn) - 1];
+		if ((entry_grade < min_runq_grade) || (entry_grade > max_runq_grade)) {
+		    continue;	/* outside grade range */
+		}
+	    }
+
 	    /* add to the list of files */
 	    if (mc >= mv_size - 1) {
 		/* note: we allow, in advance, for the last NULL entry */
