@@ -1,4 +1,4 @@
-/* @(#)src/smtprecv.c	1.16 9/6/92 01:33:48 */
+/* @(#) $Id: smtprecv.c,v 1.19 1994/10/24 13:12:25 nm4 Exp $ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -28,9 +28,20 @@
 # include "exitcodes.h"
 #endif
 
+#ifdef HAVE_RFC1413
+/* Include the declarations for ident protocol lookups */
+#include <ident.h>
+#endif /* HAVE_RFC1413 */
+
 /* library functions */
 extern long time();
 
+/* Declare the ident variables - even if HAVE_RFC1413 is not defined, so that configs
+ * can be kept consistant
+ */
+char * ident_sender = NULL;		/* The calculated identity of the sender */
+char * ident_method = NULL;		/* Method used to get identity */
+
 /* types local to this file */
 enum e_smtp_commands {
     HELO_CMD,				/* HELO domain */
@@ -120,7 +131,7 @@ receive_smtp(in, out)
     int save_dont_deliver = dont_deliver;
     FILE *save_errfile = errfile;
     int save_debug = debug;
-    int temp, i;
+    int temp, i, c;
 
     /* initialize state */
     initialize_state();
@@ -150,16 +161,46 @@ receive_smtp(in, out)
 			  (char *)NULL, (char *)NULL);
 	while (*s) {
 	    fprintf(out, "220%c", index(s, '\n') == NULL? ' ': '-');
-	    while (*s) {
-		putc(*s, out);
-		if (*s++ == '\n') break;
+	    while (c = *s) {
+		s++;
+		if (c == '\n') {
+		    break;
+		}
+		putc(c, out);
 	    }
+	    putc('\r', out);
+	    putc('\n', out);
 	}
-	putc('\r', out);
-	putc('\n', out);
 	fflush(out);
     }
 
+#ifdef HAVE_RFC1413
+    /* 
+     * This code does a Ident/RFC1413 lookup on the connecting user (if possible)
+     * Its here because this may cause a delay and it seems better to have the
+     * delay once the connection is established rather than at the start
+     *
+     * A call to ident_lookup() could be used to obtain additional info
+     * available from an RFC1413 ident call, but I don't know what we would
+     * do with the info, so don't bother to obtain it!
+     */
+    if (rfc1413_query_timeout > 0) {	/* switch off RFC1413 by setting timeout <= 0 */
+	static char ident_str[32];
+	char * ident_ptr;
+
+	if (ident_ptr = ident_id(fileno(in), rfc1413_query_timeout)) {
+	    strncpy(ident_str, ident_ptr, sizeof(ident_str));
+	    ident_str[sizeof(ident_str) - 1] = '\0';
+	    ident_sender = ident_str;
+	    free(ident_ptr);
+	    ident_method = "rfc1413";
+	} else {
+	    ident_sender = NULL;
+	    ident_method = NULL;
+	}
+    }
+#endif /* HAVE_RFC1413 */
+
     while (! term_signal || out == NULL) {
 	if (out) {
 	    alarm(smtp_receive_command_timeout);
@@ -309,16 +350,20 @@ receive_smtp(in, out)
 		    fprintf(out, "451 Failed to queue message: %s: %s\r\n",
 			    error, strerrno());
 		    fflush(out);
-		    break;
 		} else if (errfile) {
 		    fprintf(errfile, "Failed to queue message: %s: %s\r\n",
 			    error, strerrno());
 		}
+		unlink_spool();
+		reset_state();
+		smtp_remove_on_timeout = 0;
+		break;
 	    }
 	    smtp_processing_signals();
 	    if (sender == NULL) {
 		unlink_spool();
 		reset_state();
+		smtp_remove_on_timeout = 0;
 		break;
 	    }
 	    if (read_message() == NULL) {
@@ -328,6 +373,8 @@ receive_smtp(in, out)
 		    fprintf(out, "451 error in spooled message\r\n");
 		    fflush(out);
 		}
+		reset_state();
+		smtp_remove_on_timeout = 0;
 		break;
 	    }
 	    alarm(0);
@@ -733,7 +780,7 @@ smtp_receive_timeout_sig(sig)
     fprintf(out_file, "421 %s SMTP command timeout, closing channel\r\n",
 	    primary_name);
     write_log(LOG_SYS, "SMTP connection timeout%s%s.",
-	      sender_host? "while talking with": "",
+	      sender_host? " while talking with ": "",
 	      sender_host? sender_host: "");
     if (smtp_remove_on_timeout) {
 	unlink_spool();
