@@ -1,4 +1,4 @@
-/* @(#)src/spool.c	1.8 7/11/92 11:51:10 */
+/* @(#) $Id: spool.c,v 1.8 1992/07/11 11:51:10 tron Exp $ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -108,7 +108,7 @@ static int copy_old_spool();
 static void log_message();
 
 /* functions imported from libc */
-long lseek();
+/* long lseek(); */
 struct tm *localtime();
 struct tm *gmtime();
 char *ctime();
