@@ -1,4 +1,4 @@
-/* @(#)src/string.c	1.14 9/20/92 12:55:06 */
+/* @(#) $Id: string.c,v 1.14 1992/09/20 12:55:06 tron Exp $ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -197,7 +197,7 @@ char *
 strerrno()
 {
     static char misc_err[50];		/* used when sprintf must be used */
-    extern char *sys_errlist[];		/* list of error strings */
+/*  extern char *sys_errlist[];		/* list of error strings */
     extern int sys_nerr;		/* number of entries in sys_errlist */
 
     if (errno > sys_nerr || errno < 0) {
