@@ -1,4 +1,4 @@
-/* @(#)src/sysdep.c	1.31 9/9/92 22:30:30 */
+/* @(#) $Id: sysdep.c,v 1.34 1994/10/26 16:56:12 nm4 Exp $ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -121,7 +121,7 @@ static void unlock_file_by_name();
 static int lockfile_name();
 
 /* imported library functions */
-extern long lseek();
+/* extern long lseek(); */
 extern long time();
 extern char *getenv();
 extern char *getlogin();
@@ -395,6 +395,7 @@ FILE *
 fopen_as_user(fn, type, check_path, uid, gid, mode)
     char *fn;				/* name of file to open */
     char *type;				/* mode for fopen ("r", "w" or "a") */
+    int check_path;		        /* do we check all file accesses */
     int uid;				/* user id */
     int gid;				/* group id */
     int mode;				/* mode for creat() */
@@ -473,30 +474,28 @@ fopen_as_user(fn, type, check_path, uid, gid, mode)
      * NOTE:  namei calls are not known to be cheap, though on a system
      *	      with a good namei cache, this may not be too bad.
      */
-    if (check_path) {
-	fnp = fn;
-	bp = fn_build = xmalloc(strlen(fn));
-	while (new_fnp = index(fnp, '/')) {
-	    while (fnp <= new_fnp) {
-		*bp++ = *fnp++;
-	    }
-	    bp[0] = '\0';
-	    /* since the name ends in /, it can only match directories */
-	    if (stat(fn_build, &dir_stat) < 0) {
-		/* directory doesn't exist, or we can't access it */
-		return NULL;
-	    }
-	    /*
-	     * should probably use some #define somewhere.  However,
-	     * the octal numbers below are permission mode bits.
-	     */
-	    if (! ((dir_stat.st_uid == uid && (dir_stat.st_mode & 0100)) ||
-		   (dir_stat.st_gid == gid && (dir_stat.st_mode & 0010)) ||
-		   (dir_stat.st_mode & 0001)) )
-	    {
-		errno = EACCES;
-		return NULL;
-	    }
+    fnp = fn;
+    bp = fn_build = xmalloc(strlen(fn));
+    while (new_fnp = index(fnp, '/')) {
+	while (fnp <= new_fnp) {
+	    *bp++ = *fnp++;
+	}
+	bp[0] = '\0';
+	/* since the name ends in /, it can only match directories */
+	if (stat(fn_build, &dir_stat) < 0) {
+	    /* directory doesn't exist, or we can't access it */
+	    return NULL;
+	}
+	/*
+	 * should probably use some #define somewhere.  However,
+	 * the octal numbers below are permission mode bits.
+	 */
+	if (! ((dir_stat.st_uid == uid && (dir_stat.st_mode & 0100)) ||
+	       (dir_stat.st_gid == gid && (dir_stat.st_mode & 0010)) ||
+	       (dir_stat.st_mode & 0001)) )
+	{
+	    errno = EACCES;
+	    return NULL;
 	}
     }
 
@@ -1021,8 +1020,19 @@ char *
 compute_hostname()
 {
     static struct utsname utsname;
+    char * p;
 
     (void) uname(&utsname);
+
+    /*
+     * Some systems return the entire hostname, including the domain.
+     * In this case, the primary hostname should be set to this name,
+     * though for now, just truncate off the domain.
+     */
+    if ((p = index(utsname.nodename, '.')) != NULL) {
+	*p = '\0';
+    }
+
     /* Is the sysname tag used for something interesting? */
     return utsname.nodename;
 }
@@ -1162,8 +1172,10 @@ open_child(argv, envp, in, out, errfd, flags, uid, gid)
     }
     if (out && pipe(stdout_fd) < 0) {
 	/* free resources */
-	(void) close(stdin_fd[0]);
-	(void) close(stdin_fd[1]);
+	if (in) {
+	    (void) close(stdin_fd[0]);
+	    (void) close(stdin_fd[1]);
+	}
 	return EOF;
     }
 
@@ -1181,10 +1193,14 @@ open_child(argv, envp, in, out, errfd, flags, uid, gid)
     }
     if (pid < 0) {
 	/* free resources */
-	(void) close(stdin_fd[0]);
-	(void) close(stdin_fd[1]);
-	(void) close(stdout_fd[0]);
-	(void) close(stdout_fd[1]);
+	if (in) {
+	    (void) close(stdin_fd[0]);
+	    (void) close(stdin_fd[1]);
+	}
+	if (out) {
+	    (void) close(stdout_fd[0]);
+	    (void) close(stdout_fd[1]);
+	}
 	return EOF;
     }
 
@@ -1210,6 +1226,7 @@ open_child(argv, envp, in, out, errfd, flags, uid, gid)
 		{
 		    continue;
 		}
+		break;
 	    }
 	    (void) dup2(errfd, new_errfd);
 	    (void) close(errfd);
@@ -1602,11 +1619,13 @@ touch(fname, atime, mtime)
     time_t atime, mtime;
 {
     struct utimbuf times;
+    time_t cur_time;
 
     bzero((char *)&times, sizeof(times));
-    times.actime = atime;
-    times.modtime = mtime;
-    return utime(fname, (atime && mtime) ? &times: (struct utimbuf *)NULL);
+    cur_time = time(NULL);
+    times.actime  = atime ? atime : cur_time;
+    times.modtime = mtime ? mtime : cur_time;
+    return utime(fname, &times);
 }
 
 /* fsetsize - set file size
